/*
 *	File Name: alarmDialog.cpp
 *	Function: X-Windows Code to present alarm dialog to user
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qdialog.h>
#include <qt/qpushbutton.h>
#include <qt/qspinbox.h>
#include <qt/qradiobutton.h>
#include <qt/qlabel.h>

#include "radioConfig.h"
#include "hrSpinBox.h"
#include "minSpinBox.h"
#include "prgSpinBox.h"

class alarmDialog : public QDialog
{

	Q_OBJECT
	
public:
    	alarmDialog( radioConfig *config=0, QWidget *parent=0, const char *name=0 );
    	unsigned char showAlarmHr(void);
    	unsigned char showAlarmMin(void);
    	unsigned char showAlarmProg(void);
    	unsigned char showAlarmOn(void);
    	
private slots:
	void doAlarmHr(int);
	void doAlarmMin(int);
	void doAlarmProg(int);
	void doAlarmOn(bool);
		
	    
private:
	unsigned char alarmHr;
	unsigned char alarmMin;
	unsigned char alarmProg;
	unsigned char alarmOn;
};


void alarmDialog::doAlarmOn(bool value)
{
		alarmOn= (unsigned char) value;
}

void alarmDialog::doAlarmHr(int value)
{
	alarmHr= (unsigned char) value;
}

void alarmDialog::doAlarmMin(int value)
{
	alarmMin=(unsigned char) value;
}

void alarmDialog::doAlarmProg(int value)
{
	alarmProg=(unsigned char) value;
}


unsigned char alarmDialog::showAlarmHr(void)
{
	return alarmHr;
}

unsigned char alarmDialog::showAlarmMin(void)
{
	return alarmMin;
}

unsigned char alarmDialog::showAlarmProg(void)
{
	return alarmProg;
}


unsigned char alarmDialog::showAlarmOn(void)
{
	return alarmOn;
}


alarmDialog::alarmDialog( radioConfig *config, QWidget *parent, const char *name  )
        : QDialog( parent, name, TRUE )
{
		
	QPushButton *ok, *cancel;
	QRadioButton *alarmOnButt;
	hrSpinBox *hr;
	minSpinBox *min;
	prgSpinBox *prg;
	QLabel *hrLbl, *minLbl, *prgLbl;
	
	alarmHr=config->showAlarmHr();
	alarmMin=config->showAlarmMin();
	alarmProg=config->showAlarmProg();
	alarmOn=config->showAlarmOn();
		
	this->setCaption("Alarm Setting Dialog Window");

	ok = new QPushButton("OK", this);
	connect(ok, SIGNAL(clicked()), SLOT(accept()) );
	cancel = new QPushButton("Cancel", this);	
	connect(cancel, SIGNAL(clicked()), SLOT(reject()) );
	
	hr=new hrSpinBox(alarmHr, this);
	min= new minSpinBox(alarmMin, this);
	prg = new prgSpinBox(alarmProg, this);
	
	alarmOnButt = new QRadioButton("Alarm ON/OFF", this);
	alarmOnButt->setChecked(config->showAlarmOn()); 
	
	connect(hr, SIGNAL(valueChanged(int)), SLOT(doAlarmHr(int)) );
	connect(min, SIGNAL(valueChanged(int)), SLOT(doAlarmMin(int)) );
	connect(prg, SIGNAL(valueChanged(int)), SLOT(doAlarmProg(int)) );
	connect(alarmOnButt, SIGNAL(toggled(bool)), SLOT(doAlarmOn(bool)) );
	
	hrLbl = new QLabel("Hour", this);
	minLbl = new QLabel("Minute", this);
	prgLbl = new QLabel("Program", this);
	
	hrLbl->setGeometry(30, 15, 45, 15);
	minLbl->setGeometry(90, 15, 45, 15);
	prgLbl->setGeometry(150, 15, 45, 15);
	
	hr->setGeometry(30, 40, 45, 5);
	min->setGeometry(90, 40, 45, 5);
	prg->setGeometry(150, 40, 45, 5);
	alarmOnButt->setGeometry(210, 45, 120, 20);
	
	ok->setGeometry(60, 100, 80, 30);
	cancel->setGeometry(180, 100, 80, 30);
 
	setMinimumSize(320,150);
	setMaximumSize(320,150);

}

