/*
 *	File Name: lcd.cpp
 *	Function: X-Windows Code to present the LCD like display on the GUI.
 *                This part of the code is the most complex!
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qlcdnumber.h>
#include <qt/qfont.h>
#include <qt/qcolor.h>
#include <qt/qpalette.h>
#include <qt/qlayout.h>
#include <qt/qlabel.h>

#include "tunerCard.h"
#include "radioConfig.h"

#include <sys/timeb.h>
#include <time.h>
#include <string.h>

class LCDWidget : public QWidget
{

	Q_OBJECT
	
public:
    LCDWidget( tunerCard *cardValue=0, radioConfig *configuration=0, QLabel *status=0, QWidget *parent=0, const char *name=0 );
    void setOnOff(void);
    void unsetOnOff(void);
    void unsetAlarmMin(void);
    void setSleepTime(unsigned char);
    void setMem(bool);
    
public slots:
    void incrementValue(void);
    
private:    
    QLCDNumber  *freqDisplay;
    QLCDNumber  *timeDisplay;
    tunerCard *card;
    radioConfig *config;
    QLabel *statusBar;
    bool onOff, doneAlarmMin, sleepTimerFlag, mem;
    unsigned char sleepTime; 
};


void LCDWidget::setSleepTime(unsigned char value)
{
	sleepTimerFlag=TRUE;
	time_t tt=time(NULL);
	tm *tmStruct=localtime(&tt);
	sleepTime=((tmStruct->tm_min) + value) % 60;
}


void LCDWidget::setMem(bool value)
{
	mem=value;
}


void LCDWidget::setOnOff(void)
{
	onOff=TRUE;
}

void LCDWidget::unsetOnOff(void)
{
	onOff=FALSE;
}

void LCDWidget::unsetAlarmMin(void)
{
	doneAlarmMin=FALSE;
}
    

/* incrementValue function, is called to refresh the LCD GUI display every few millisecs
 * included in this function are some band limiting logic to prevent the frequency of FMR2 from over or underrun
 */

void LCDWidget::incrementValue( void )
{
	char tmDisplay[10], statusDisp[40];
	int tmBuffer;	
      
        tmDisplay[9]=0;
	card->doRead(); /*Attempt to read FMR@ divider ratio */

	/* Band limiting logic to prevent frequency over or underrun */	

	if(card->showFreq() < 88000)
	{
		card->setFreq(88000);
	} else 
	if (card->showFreq() > 108000)
	{
		card->setFreq(108000);
	} else {
	
		tmBuffer=card->showApproxFreq();
		
		int tmpBig=tmBuffer/100;
		int tmpSmall=tmBuffer%100;
		
		if(tmpBig /100 != 0)
		{
			tmDisplay[0]=char (tmpBig / 100 + 0x30);
			tmpBig-=100;
		}
		else
			tmDisplay[0]=' ';
			
		tmDisplay[1]=char (tmpBig / 10 + 0x30);
		tmDisplay[2]=char (tmpBig % 10 + 0x30);		
		tmDisplay[4]=char (tmpSmall / 10 + 0x30);
		tmDisplay[5]=char (tmpSmall % 10 + 0x30);
		tmDisplay[3]='.';
		tmDisplay[6]=0;		
		
		freqDisplay->display(tmDisplay);
			
		time_t tt=time(NULL);

		tm *tmStruct=localtime(&tt);
	
		tmDisplay[0]=char ((tmStruct->tm_hour) / 10 + 0x30);
		tmDisplay[1]=char ((tmStruct->tm_hour) % 10 + 0x30);
		tmDisplay[3]=char ((tmStruct->tm_min) / 10 + 0x30);
		tmDisplay[4]=char ((tmStruct->tm_min) % 10 + 0x30);
		tmDisplay[2]=':';
	        tmDisplay[5]=' ';
	        tmDisplay[6]=' ';
	        if(((card->showProgNo()) / 10) != 0)
			tmDisplay[7]=char ((card->showProgNo()) / 10 + 0x30);
		else
			tmDisplay[7]=' ';
		tmDisplay[8]=char ((card->showProgNo()) % 10 + 0x30); 
	
		timeDisplay->display(tmDisplay);


		/* If need to do the sleep function do it! */
		if ((sleepTimerFlag==TRUE) && ((tmStruct->tm_min) == sleepTime))
		{
			sleepTimerFlag=FALSE;
			card->setMute();
		}
		
	        /* If need to do the alarm function do it! */
		if((config->showAlarmOn()) && (doneAlarmMin==FALSE))
		{
			if((unsigned int) (tmStruct->tm_hour) == (config->showAlarmHr()))
			{
				if((unsigned int) (tmStruct->tm_min) == (config->showAlarmMin()))
				{
					card->loadProg(config->showAlarmProg());
					doneAlarmMin=TRUE;
				}
			}	
		}
	

		/* Update the rest of the GUI status bar */
		if(card->isMuted() == 1)
			strcpy(statusDisp, "Muted");
		else
		{
			if(card->showStereo() == 1)
				strcpy(statusDisp, "Mono  ");
			else
				strcpy(statusDisp, "Stereo");
			strcat(statusDisp, " Normal");
		}	
	
		if(card->isTuned() == 0)
			strcat(statusDisp, " Tuned");
			
		if(mem)
			strcat(statusDisp, " Memory");
		
		if(config->showAlarmOn())
			strcat(statusDisp, " Alarm On");	
		statusBar->setText(statusDisp);
		
		if(sleepTimerFlag==TRUE)
			strcat(statusDisp, " Sleep");	
		statusBar->setText(statusDisp);
		
	}

}


LCDWidget::LCDWidget( tunerCard *cardValue, radioConfig *configuration, QLabel *status=0, QWidget *parent, const char *name  )
        : QWidget( parent, name )
{
    card=cardValue;
    config=configuration;
    statusBar=status;
    sleepTimerFlag=FALSE;
    mem=FALSE;
    
    QColorGroup colGrp(green, black, green, green, green, green, white);

    freqDisplay  = new QLCDNumber( 6, this, "freqDisplay" );
    freqDisplay->setPalette(QPalette(colGrp, colGrp, colGrp));
    freqDisplay->setSegmentStyle(QLCDNumber::Filled);
    freqDisplay->setFrameStyle(QFrame::Panel| QFrame::Plain);
    freqDisplay->setLineWidth(0);
    
    timeDisplay  = new QLCDNumber( 9, this, "timeDisplay" );
    timeDisplay->setPalette(QPalette(colGrp, colGrp, colGrp));
    timeDisplay->setSegmentStyle(QLCDNumber::Filled);
    timeDisplay->setFrameStyle(QFrame::Panel| QFrame::Plain);
    timeDisplay->setLineWidth(0);

    QLabel *lbl = new QLabel("MHz", this);
    lbl->setAlignment(AlignCenter);
    lbl->setPalette(QPalette(colGrp, colGrp, colGrp));
    lbl->setFont(QFont("Helvetica", 24, QFont::Bold));

    freqDisplay->setGeometry ( 0, 0, 100, 50);
    timeDisplay->setGeometry  (0, 50, 150, 25);
    lbl->setGeometry(100, 0, 50, 50);
    


}

