/*
 *	File Name: main.cpp
 *	Function: Main X-Windows Code to kick off the GUI application
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */

#include <qt/qapplication.h>
#include <qt/qtimer.h>
#include <qt/qwidget.h>
#include <qt/qlabel.h>
#include <qt/qtooltip.h>

#include "lcd.h"
#include "buttons.h"
#include "progButtons.h"
#include "tunerCard.h"
#include "radioConfig.h"
#include "volSlider.h"
#include "exitDialog.h"

class MyWidget : public QWidget
{
public:
    MyWidget( QWidget *parent=0, const char *name=0 );
    ~MyWidget(void);
    bool MyWidget::close(bool forceKill= FALSE);
    

private:
    radioConfig *config;
    tunerCard *card;
    QLabel *statusBar;
    QTimer *timer;
    LCDWidget *lcd;
    ButtonsWidget *butt;
    progButtonsWidget *progButt;
    volSlider *slider;
};

bool MyWidget::close(bool forceKill)
{
	if(forceKill)
		return QWidget::close(forceKill);
	else
	{
		exitDialog *ext = new exitDialog(card, this);
	
		int resultCode = ext->exec();

    		if ( resultCode ) 
    			return QWidget::close(forceKill);
		else
			return false;
	}

}

MyWidget::~MyWidget(void)
{    	
    	config->commitConfig();
}

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{

    setFixedSize(775, 130);
    
    setCaption("PCR Radio Application by MediaForte");
    config=new radioConfig;

    card=new tunerCard(config);

    slider = new volSlider(config, this);
    statusBar= new QLabel(this);
    timer = new QTimer(this);
    lcd = new LCDWidget( card, config, statusBar, this);
    butt = new ButtonsWidget (slider, card, config, lcd, timer, this);
    progButt = new progButtonsWidget(card, butt, this);
    
    QColorGroup colGrp(yellow, black, yellow, yellow, yellow, yellow, white);
    statusBar->setPalette(QPalette(colGrp, colGrp, colGrp));
     
    lcd->unsetAlarmMin();
    
    card->loadProg(1);
    
    lcd->connect(timer, SIGNAL(timeout()), lcd, SLOT(incrementValue()) );
    timer->start(200, FALSE);
    
    statusBar->setGeometry ( 0, 130-15, 270, 15);
    lcd->setGeometry ( 0, 0, 150, 75);
    progButt->setGeometry( 775-625, 0, 625, 75);
    slider->setGeometry( 775-500, 130-15, 500, 15);
    butt->setGeometry(0, 130-15-35, 775, 35);   

    QToolTip::add(slider, "Volume Control");

}

int main( int argc, char **argv )
{ 
    QApplication a( argc, argv );
    MyWidget *f= new MyWidget;    
    f->setGeometry( 0, 0, 775, 130 );
    a.setMainWidget(f);
    f->show();     

    a.exec();
        
    return 0;
}
