/*
 * pcimod.c - Module PCI driver for SF16-pcr
 *
 * Limitations:
 *   -it only runs on intel platforms.
 *	  -it will only compile correctly for kernel 2.2.15
 *	  - pcr IO port is hardwired to 0x384, module driver might mess up any
 *		 device which may happen to occupy the same address.
 *
 * Tested with 2.2.15 on the x86
 *
  *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 */


#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/delay.h>

#define __NO_VERSION__ /* don't define kernel_version in module.h */
#include <linux/module.h>
#include <linux/version.h>

#include <asm/system.h> /* cli(), *_flags */
#include <linux/kernel.h> /* printk */
#include <linux/errno.h> /* error codes */
#include "sysdep-2.1.h"
#include "pcr.h"


int check_EEPROM(void)
{
	char array[4];
	int err;

	if((err=read_EEPROM(&array[0])) != 0)
		return err;

	if((err=check_EEPROM_words(&array[0])) != 0)
		return err;

	return 0;
}


int check_EEPROM_words(char *ptr)
{
	if(*ptr != 0x01)
		return 13;
	ptr++;
	
	if(*ptr != 0x00)
		return 14;

	ptr++;

	if(*ptr != 0x01)
		return 15;
	ptr++;

	if(*ptr != 0x00)
		return 16;
	return 0;
}


int read_EEPROM(char *ptr)
{
	send_START();
	send_DEVICE_address_WRITE();
	if(check_ack())
		return 17;
	send_EEPROM_address();
	if(check_ack())
		return 18;
	send_START();
	send_DEVICE_address_READ();
	if(check_ack())
		return 19;
	read_BYTE(ptr);
	ptr++;
	read_BYTE(ptr);
	ptr++;
	read_BYTE(ptr);
	ptr++;
	read_LASTBYTE(ptr);
	outb(0x00, I2C_PORT);
	return 0;
	
}

void read_LASTBYTE(char *ptr)
{
	int i, temp, result=0;

	for(i=0; i <8; i++)
	{
		outb(0x02, I2C_PORT);
		udelay(4);
		temp=inb(I2C_PORT) & 0x80;
		if(temp == 0)
			send_ZERO();
		else
			send_ONE();
		temp>>=i;
		result|=temp;
	}

	send_ONE();
	outb(0x0c, I2C_PORT);
	udelay(4);
	outb(0x0d, I2C_PORT);
	udelay(4);
	outb(0x0f, I2C_PORT);
	udelay(4);

	*ptr= (char) result;

}

void read_BYTE(char *ptr)
{
	int i, temp, result=0;

	for(i=0; i <8; i++)
	{
		outb(0x02, I2C_PORT);
		udelay(4);
		temp=inb(I2C_PORT) & 0x80;
		if(temp == 0)
			send_ZERO();
		else
			send_ONE();
		temp>>=i;
		result|=temp;
		
	}

        outb(0x0c, I2C_PORT);
	udelay(4);
	outb(0x0d, I2C_PORT);
	udelay(4);
	outb(0x0c, I2C_PORT);
	udelay(4);

	*ptr= (char) result;
}

void send_START(void)
{
	outb(0x0f, I2C_PORT);
	udelay(4);
	outb(0x0d, I2C_PORT);
	udelay(4);
	outb(0x0c, I2C_PORT);
	udelay(4);	
}

int check_ack(void)
{
	int temp, i;

	outb(0x02, I2C_PORT);
	for(i = 0; i < 10000; i++)
	{
		temp=inb(I2C_PORT) & 0X80;
		if(temp == 0)
			break;
	}
	if(temp != 0)
		return 99;
	udelay(4);
	outb(0x0c, I2C_PORT);
	udelay(4);
	outb(0x0d, I2C_PORT);
	udelay(4);
	outb(0x0c, I2C_PORT);
	udelay(4);
	return 0;
}

void send_EEPROM_address(void)
{
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ONE();
	send_ONE();
	send_ZERO();
	send_ZERO();
	
}

void send_DEVICE_address_READ(void)
{
	send_ONE();
	send_ZERO();
	send_ONE();
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ONE();

}

void send_DEVICE_address_WRITE(void)
{
	send_ONE();
	send_ZERO();
	send_ONE();
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ZERO();
	send_ZERO();

}

void send_ONE(void)
{
	outb(0x0e, I2C_PORT);
	udelay(4);
	outb(0x0f, I2C_PORT);
	udelay(4);
	outb(0x0e, I2C_PORT);
	udelay(4);
	
}

void send_ZERO(void)
{
	outb(0x0c, I2C_PORT);
	udelay(4);
	outb(0x0d, I2C_PORT);
	udelay(4);
	outb(0x0c, I2C_PORT);
	udelay(4);

}



